"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshChannelClosedEventArgs = void 0;
class SshChannelClosedEventArgs {
    constructor(exitStatusOrSignalOrError, errorMessage) {
        if (typeof exitStatusOrSignalOrError === 'number') {
            this.exitStatus = exitStatusOrSignalOrError;
        }
        else if (typeof exitStatusOrSignalOrError === 'string') {
            this.exitSignal = exitStatusOrSignalOrError;
            this.errorMessage = errorMessage;
        }
        else if (exitStatusOrSignalOrError instanceof Error) {
            this.error = exitStatusOrSignalOrError;
        }
    }
}
exports.SshChannelClosedEventArgs = SshChannelClosedEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-ssh-events-sshChannelClosedEventArgs.js.map