"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeRandom = void 0;
const crypto = require("crypto");
class NodeRandom {
    getBytes(buffer) {
        const randomBytes = crypto.randomBytes(buffer.length);
        randomBytes.copy(buffer);
    }
}
exports.NodeRandom = NodeRandom;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-ssh-algorithms-node-nodeRandom.js.map