"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshExtendedDataEventArgs = exports.SshExtendedDataType = void 0;
var SshExtendedDataType;
(function (SshExtendedDataType) {
    /**
     * The extended data type SSH_EXTENDED_DATA_STDERR has been defined for stderr data.
     */
    SshExtendedDataType[SshExtendedDataType["STDERR"] = 1] = "STDERR";
})(SshExtendedDataType = exports.SshExtendedDataType || (exports.SshExtendedDataType = {}));
class SshExtendedDataEventArgs {
    constructor(dataTypeCode, data) {
        this.dataTypeCode = dataTypeCode;
        this.data = data;
    }
    toString() {
        return `${SshExtendedDataType[this.dataTypeCode]}: ${this.data.toString()}`;
    }
}
exports.SshExtendedDataEventArgs = SshExtendedDataEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-ssh-events-sshExtendedDataEventArgs.js.map