"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
// Generated from ../../../cs/src/Contracts/TunnelEvent.cs
/* eslint-disable */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TunnelEvent = exports.error = exports.warning = exports.info = void 0;
/**
 * Default event severity.
 */
exports.info = 'info';
/**
 * Warning event severity.
 */
exports.warning = 'warning';
/**
 * Error event severity.
 */
exports.error = 'error';
exports.TunnelEvent = {
    info: exports.info,
    warning: exports.warning,
    error: exports.error,
};
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-contracts-tunnelEvent.js.map