"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.RefreshingTunnelAccessTokenEventArgs = void 0;
/**
 * Event args for tunnel access token refresh event.
 */
class RefreshingTunnelAccessTokenEventArgs {
    /**
     * Creates a new instance of RefreshingTunnelAccessTokenEventArgs class.
     */
    constructor(
    /**
     * Tunnel access scope to get the token for.
     */
    tunnelAccessScope, 
    /**
     * Cancellation token that event handler may observe when it asynchronously fetches the tunnel access token.
     */
    cancellation) {
        this.tunnelAccessScope = tunnelAccessScope;
        this.cancellation = cancellation;
    }
}
exports.RefreshingTunnelAccessTokenEventArgs = RefreshingTunnelAccessTokenEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-connections-refreshingTunnelAccessTokenEventArgs.js.map