"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
// Generated from ../../../cs/src/Contracts/TunnelAuthenticationSchemes.cs
/* eslint-disable */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TunnelAuthenticationSchemes = void 0;
/**
 * Defines string constants for authentication schemes supported by tunnel service APIs.
 */
var TunnelAuthenticationSchemes;
(function (TunnelAuthenticationSchemes) {
    /**
     * Authentication scheme for AAD (or Microsoft account) access tokens.
     */
    TunnelAuthenticationSchemes["Aad"] = "aad";
    /**
     * Authentication scheme for GitHub access tokens.
     */
    TunnelAuthenticationSchemes["GitHub"] = "github";
    /**
     * Authentication scheme for tunnel access tokens.
     */
    TunnelAuthenticationSchemes["Tunnel"] = "tunnel";
    /**
     * Authentication scheme for tunnelPlan access tokens.
     */
    TunnelAuthenticationSchemes["TunnelPlan"] = "tunnelplan";
})(TunnelAuthenticationSchemes = exports.TunnelAuthenticationSchemes || (exports.TunnelAuthenticationSchemes = {}));
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-contracts-tunnelAuthenticationSchemes.js.map