"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultPortForwardMessageFactory = void 0;
const portForwardChannelOpenMessage_1 = require("./messages/portForwardChannelOpenMessage");
const portForwardRequestMessage_1 = require("./messages/portForwardRequestMessage");
const portForwardSuccessMessage_1 = require("./messages/portForwardSuccessMessage");
class DefaultPortForwardMessageFactory {
    createRequestMessageAsync(port) {
        return Promise.resolve(new portForwardRequestMessage_1.PortForwardRequestMessage());
    }
    createSuccessMessageAsync(port) {
        return Promise.resolve(new portForwardSuccessMessage_1.PortForwardSuccessMessage());
    }
    createChannelOpenMessageAsync(port) {
        return Promise.resolve(new portForwardChannelOpenMessage_1.PortForwardChannelOpenMessage());
    }
}
exports.DefaultPortForwardMessageFactory = DefaultPortForwardMessageFactory;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-ssh-tcp-portForwardMessageFactory.js.map