// Filesystem restriction configs (internal structures built from permission rules)
export {};//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@vscode-sandbox-runtime-dist-sandbox-sandbox-schemas.js.map