"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshSessionAlgorithms = void 0;
class SshSessionAlgorithms {
    dispose() {
        if (this.cipher)
            this.cipher.dispose();
        if (this.decipher)
            this.decipher.dispose();
        if (this.signer)
            this.signer.dispose();
        if (this.verifier)
            this.verifier.dispose();
    }
}
exports.SshSessionAlgorithms = SshSessionAlgorithms;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-824c4c46a288b839f13b24022655329c2aeb9f81/node_modules-@microsoft-dev-tunnels-ssh-sshSessionAlgorithms.js.map